@@ -2,7 +2,7 @@
 # ==============================================================================
 # portsreinstall library script
 # - Operations of commands as well as check of command line arguments -
-# Copyright (C) 2013 Mamoru Sakaue, MwGhennndo, All Rights Reserved.
+# Copyright (C) 2013-2014 Mamoru Sakaue, MwGhennndo, All Rights Reserved.
 # This software is distributed under the 2-Clause BSD License.
 # ==============================================================================
 
@@ -136,11 +136,11 @@ command_parse_args ()
 		obsolete)
 			;;
 		'')
-			message_echo "ERROR: Missing operation which must be \"add\" or \"del\"." >&2
+			message_echo "ERROR: Missing operation which must be \"leaves\" or \"obsolete\"." >&2
 			exit 1
 			;;
 		*)
-			message_echo "ERROR: Invalid operation [$COMMAND_OPERATION]; it must be \"add\" or \"del\"." >&2
+			message_echo "ERROR: Invalid operation [$COMMAND_OPERATION]; it must be \"leaves\" or \"obsolete\"." >&2
 			exit 1
 			;;
 		esac
@@ -198,7 +198,7 @@ command_parse_args ()
 	options)
 		_command_parse_args__chk_no_arg $#
 		;;
-	reconf|forget|escape|restore)
+	reconf|rmconf|forget|escape|restore)
 		misc_chk_privilege
 		temp_warn_obsolete_temp_db
 		_command_parse_args__chk_glob_args $#
@@ -440,7 +440,7 @@ command_forget ()
 # ============= Execute command operations which are irrespective of option settings =============
 command_exec_irrespective_of_saved_options ()
 {
-	local dbfile tmp_manually_done_diff evalated_globs dbdir_parent dbdir_node arcfile grandtitle title isfirst origin origin_regexp backup_pkg origin_orig origin_replace pkg_orig tmp_done_orig tmp_pkgsanity nlines iline pkg
+	local dbfile tmp_manually_done_diff evalated_globs dbdir_parent dbdir_node arcfile grandtitle title isfirst origin origin_regexp backup_pkg origin_orig origin_replace pkg_orig tmp_done_orig tmp_pkgsanity nlines iline pkg make_target
 	case $COMMAND_MODE in
 	ok)
 		dbfile=${DBDIR}/manually_done.list
@@ -584,15 +584,25 @@ command_exec_irrespective_of_saved_options ()
 		pkgsys_eval_ports_glob "$@"
 		exit
 		;;
-	reconf)
+	reconf|rmconf)
 		_command_exec_irrespective_of_saved_options__notify_ignored_options
-		message_echo "Temporary database sections for the following ports are reset for later rebuild:"
+		case $COMMAND_MODE in
+		reconf)
+			message_echo "Reconfigure the specified port options."
+			make_target=config
+			;;
+		rmconf)
+			message_echo "The specified port options are reset to the default."
+			make_target=rmconfig
+			;;
+		esac
+		message_echo "Affected parts of the temporary database are reset automatically."
 		for origin in `pkgsys_eval_ports_glob "$@"`
 		do
 			origin_regexp=`str_escape_regexp "$origin"`
 			if grep -q -E "^$origin_regexp$" "${DBDIR}/inspected_ports" 2> /dev/null
 			then
-				database_build_make "$origin" config
+				database_build_make "$origin" $make_target
 				database_build_patch_reconf "$origin"
 			else
 				message_echo "$origin (not inspected)"
@@ -715,28 +725,72 @@ command_exec_irrespective_of_saved_options ()
 		exit
 		;;
 	pkgsanity)
-		tmp_pkgsanity=${TMPDIR}/command_exec_irrespective_of_saved_options:pkgsanity
+		tmp_pkgs=${TMPDIR}/command_exec_irrespective_of_saved_options:pkgs
+		tmp_orgs=${TMPDIR}/command_exec_irrespective_of_saved_options:orgs
+		tmp_pkgs_for_an_org=${TMPDIR}/command_exec_irrespective_of_saved_options:pkgs_for_an_org
+		tmp_pkgs_for_an_org_msg=${TMPDIR}/command_exec_irrespective_of_saved_options:pkgs_for_an_org_msg
 		_command_exec_irrespective_of_saved_options__notify_ignored_options
 		if [ $# -eq 0 ]
 		then
 			message_echo "Sanity check of the installed files for each package:"
 			message_echo
-			pkg_info_Ea > $tmp_pkgsanity
+			pkg_info_Ea > $tmp_pkgs
+			pkg_info_qoa > $tmp_orgs
 		else
 			message_echo "Examining the installed files for each specified package:"
 			message_echo
-			pkgsys_eval_ports_glob "$@" | while read origin
+			pkgsys_eval_ports_glob "$@" > $tmp_orgs
+			while read origin
 			do
 				pkg_info_qO "$origin" || :
-			done | grep -v '^[[:space:]]*$' > $tmp_pkgsanity
+			done < $tmp_orgs | grep -v '^[[:space:]]*$' > $tmp_pkgs
 		fi
-		nlines=`wc -l < $tmp_pkgsanity`
+		message_echo "<< Phase 1: Check and fix duplicated packages registrations for the same port origin >>"
+		message_echo
+		while read origin
+		do
+			[ `pkg_info_qO "$origin" | wc -l` -gt 1 ] || continue
+			pkg_info_qO "$origin" | while read pkg
+			do
+				echo "$((0+`pkg_check_sanity \"$pkg\" | wc -l`))" "$pkg"
+			done | sort -g > $tmp_pkgs_for_an_org
+			pkg_valid=`head -n 1 "$tmp_pkgs_for_an_org" | cut -d ' ' -f 2`
+			message_echo "Port [$origin] has multiply registered packages."
+			message_echo "The valid one will be [$pkg_valid]."
+			message_echo "Invalid one(s) will be as follows and to be deleted:"
+			sed 1d "$tmp_pkgs_for_an_org" | cut -d ' ' -f 2 > $tmp_pkgs_for_an_org_msg
+			message_cat "$tmp_pkgs_for_an_org_msg"
+			errout=/dev/stderr
+			[ "x$opt_batch_mode" = xyes ] && errout=/dev/null
+			if backup_pkg=`pkgsys_create_backup_pkg "$pkg_valid" "${DBDIR}/backup_packages"`
+			message_echo
+			then
+				pkg_delete_f `cat "$tmp_pkgs_for_an_org"` 2> $errout || {
+					message_echo "WARNING: Deletion of the broken packages may not be fully successful, but continuing anyway." >&2
+					message_echo >&2
+				}
+				pkg_add_fF "$backup_pkg" || {
+					message_echo "WARNING: Reinstallation of the most valid package failed, but continuing anyway." >&2
+					message_echo >&2
+				}
+			else
+				message_echo "WARNING: Backup of the most valid package failed, but continuing anyway." >&2
+				pkg_delete_f `cat "$tmp_pkgs_for_an_org"` 2> $errout || {
+					message_echo "WARNING: Deletion of the broken packages may not be fully successful, but continuing anyway." >&2
+					message_echo >&2
+				}
+			fi
+		done < $tmp_orgs
+		
+		message_echo "<< Phase 2: Check and mark broken packages for reinstallation >>"
+		message_echo
+		nlines=`wc -l < $tmp_pkgs`
 		iline=1
 		while [ $iline -le $nlines ]
 		do
-			pkg=`sed -n ${iline}p "$tmp_pkgsanity"`
+			pkg=`sed -n ${iline}p "$tmp_pkgs"`
 			iline=$((${iline}+1))
-			origin=`pkg_info_qo "$pkg"`
+			origin=`pkg_info_qo "$pkg"` || continue
 			[ -n "$origin" ] || continue
 			origin_regexp=`str_escape_regexp "$origin"`
 			grep -q -E "^$origin_regexp$" "${DBDIR}/damaged_package" 2>/dev/null && continue
@@ -803,11 +857,13 @@ command_exec_before_tools_upgrade ()
 	case $COMMAND_MODE in
 	reset)
 		message_echo "Starting to reset the temporary database by preserving the initial snapshot of installed packages..."
+		program_chk_stage_complete INSPECT_ALL_DEPENDENCIES && touch "${DBDIR}/installed_ports.inspected"
 		find "${DBDIR}" -depth 1 \
 			-not \( -name saved_options.sh \
 				-or -name initial -or -name MYVERSION \
 				-or -name backup_failure -or -name installed_ports\* \) \
 			-exec rm -rf {} \; 2> /dev/null || :
+		touch "${DBDIR}/MYVERSION"
 		case $COMMAND_OPERATION in
 		all)
 			command_exec_without_pkgtools__notify_reset_options
@@ -936,16 +992,19 @@ command_exec_before_building_tempdb ()
 # ============= Execute command operations which must be done before actual (re/de)installation processes =============
 command_exec_before_actual_re_de_installation ()
 {
+	local tmp_msg
 	case $COMMAND_MODE in
 	do|redo)
 		case $COMMAND_DO_MODE in
 		prepare)
-			message_cat 3<< eof
+			tmp_msg=${TMPDIR}/command_exec_before_actual_re_de_installation:msg
+			cat > "$tmp_msg" << eof
 Done (skipped reinstallation) at `message_timestamp`
 
  You can restart this process from the aborted/terminated point by executing without options or arguments as:
   ${APPNAME}
 eof
+			message_cat "$tmp_msg"
 			temp_terminate_process ()
 			{
 			}
