@@ -2,13 +2,20 @@
 # ==============================================================================
 # portsreinstall library script
 # - Parsing of configuration files -
-# Copyright (C) 2013 Mamoru Sakaue, MwGhennndo, All Rights Reserved.
+# Copyright (C) 2013-2014 Mamoru Sakaue, MwGhennndo, All Rights Reserved.
 # This software is distributed under the 2-Clause BSD License.
 # ==============================================================================
 
 # ============= Set up environment variables =============
 conf_setup_ports_envs ()
 {
+	local tmp_work
+	tmp_work=${TMPDIR}/conf_setup_ports_envs:work
+	rm -rf "$tmp_work"
+	mkdir "$tmp_work"
+	PORTSDIR=${PORTSDIR:-`[ -e /etc/make.conf ] && make -C "$tmp_work" -f /etc/make.conf -V PORTSDIR 2> /dev/null`} || :
+	PORTSDIR=${PORTSDIR:-/usr/ports}
+	pkgsys_chk_ports_tree_implementation
 	PORTS_MOVED_DB=${PORTSDIR}/MOVED
 	PORTS_INDEX_DB=${PORTSDIR}/INDEX-`uname -r | cut -d . -f 1`
 	if [ ! -e "${PORTS_INDEX_DB}" ]
@@ -23,14 +30,9 @@ conf_setup_ports_envs ()
 }
 
 # ============= Set up environment variables =============
-conf_setup_envs ()
+conf_setup_packages_envs ()
 {
-	local tmp_work
-	tmp_work=${TMPDIR}/conf_setup_envs:work
-	rm -rf "$tmp_work"
-	mkdir "$tmp_work"
-	PORTSDIR=${PORTSDIR:-`[ -e /etc/make.conf ] && make -C "$tmp_work" -f /etc/make.conf -V PORTSDIR 2> /dev/null`} || :
-	PORTSDIR=${PORTSDIR:-/usr/ports}
+	local var
 	for var in LOCALBASE PORT_DBDIR LINUXBASE DISTDIR PACKAGES PKGREPOSITORYSUBDIR PKGREPOSITORY
 	do
 		val=`pkgsys_sysvar $var`
@@ -46,7 +48,6 @@ conf_setup_envs ()
 	PACKAGECHECKSUMDIR=${PACKAGEDIR:-'pub/FreeBSD/ports/%s/packages-%s/All/'}
 	PACKAGEROOTS=${PACKAGEROOTS:-${PACKAGECHECKSUMROOTS}}
 	PACKAGEDIR=${PACKAGEDIR:-${PACKAGECHECKSUMDIR}}
-	conf_setup_ports_envs
 }
 
 # ============= Parse configuration variables given for each port glob =============
@@ -206,7 +207,7 @@ conf_manipulate_available_var_defs ()
 {
 	(set -e
 		tmpfile_unsetvars=${TMPDIR}/conf_manipulate_available_var_defs::unsetvars.sh
-		_CONFVARS='ENV ALT_MOVED HOLD TABOO REPLACE_FROM REPLACE_TO MARG_TARGET MARG_DEF MENV_TARGET MENV_DEF BEFOREBUILD BEFOREDEINSTALL AFTERINSTALL'
+		_CONFVARS='ENV ALT_MOVED HOLD TABOO NOPKG REPLACE_FROM REPLACE_TO MARG_TARGET MARG_DEF MENV_TARGET MENV_DEF BEFOREBUILD BEFOREDEINSTALL AFTERINSTALL'
 		for item in ${_CONFVARS}
 		do
 			misc_get_all_vardefs | grep -e "^${item}_" | cut -d = -f 1 | sed 's/^/unset /'
