@@ -2,7 +2,7 @@
 # ==============================================================================
 # portsreinstall library script
 # - Operations for building the temporary database -
-# Copyright (C) 2013 Mamoru Sakaue, MwGhennndo, All Rights Reserved.
+# Copyright (C) 2013-2014 Mamoru Sakaue, MwGhennndo, All Rights Reserved.
 # This software is distributed under the 2-Clause BSD License.
 # ==============================================================================
 
@@ -98,7 +98,8 @@ database_build_register_obsolete_port ()
 		do
 			for tag in all run build
 			do
-				ln -f "${DBDIR}/initial/$origin/${table}.${tag}.${level}" "$dbpath/${table}.${tag}.${level}.src"
+				srcfile=${DBDIR}/initial/$origin/${table}.${tag}.${level}
+				[ -e "$srcfile" ] && ln -f "$srcfile" "$dbpath/${table}.${tag}.${level}.src"
 			done
 		done
 	done
@@ -214,11 +215,13 @@ database_build_setup_initial_node ()
 		do
 			origin_requirement=`pkgsys_init_pkg_orig_by_ambiguous_matching "$requirement" || :`
 			[ -n "$origin_requirement" ] && echo "$origin_requirement"
+			:
 		done > $dbpath/requirements.all.full
 		pkg_info_qR "$pkg" | while read dependent
 		do
 			origin_dependent=`pkgsys_init_pkg_orig_by_ambiguous_matching "$dependent" || :`
 			[ -n "$origin_dependent" ] && echo "$origin_dependent"
+			:
 		done > $dbpath/dependents.all.full
 		for table in dependents requirements
 		do
@@ -336,16 +339,28 @@ database_build_is_port_already_inspected_in_required_level ()
 # ============= Update the current package name of a port =============
 database_build_update_pkgname ()
 {
-	local origin pkg savefile
+	local origin pkg savefile origin_orig
 	origin=$1
 	savefile=${DBDIR}/requires/$origin/current_version
-	pkg=`pkg_info_qO "$origin" || :`
-	if [ -z "$pkg" -a -e "${DBDIR}/requires/$origin/initial_orig" ]
+	if [ -e "${DBDIR}/REVERSE_REPLACE.complete_sed_pattern" ]
 	then
-		origin_orig=`cat "${DBDIR}/requires/$origin/initial_orig"`
-		pkg=`pkg_info_qO "$origin_orig" || :`
+		origin_orig=`echo "$origin" \
+			| sed -E -f "${DBDIR}/REVERSE_REPLACE.complete_sed_pattern"` || :
+		[ "x$origin_orig" = "x$origin" ] && origin_orig=
+	else
+		origin_orig=
 	fi
-	echo -n "$pkg" > $savefile
+	echo "$origin_orig" "$origin" | tr ' ' '\n' | grep -v '^$' | while read orig
+	do
+		pkg=`pkg_info_qO "$orig" || :`
+		if [ -z "$pkg" -a -e "${DBDIR}/requires/$orig/initial_orig" ]
+		then
+			orig_init=`cat "${DBDIR}/requires/$orig/initial_orig"`
+			pkg=`pkg_info_qO "$orig_init" || :`
+		fi
+		[ -n "$pkg" ] && echo "$pkg"
+		:
+	done > $savefile
 	cat "$savefile"
 }
 
@@ -436,7 +451,7 @@ database_build_is_currentpkg_latest ()
 {
 	local origin pkg_cur pkg_new
 	origin=$1
-	pkg_cur=`database_build_get_pkgname "$origin"`
+	pkg_cur=`database_build_get_pkgname "$origin" | tr '\n' ' ' | sed 's/ *$//'`
 	pkg_new=`database_build_get_new_pkgname "$origin"`
 	[ "x$pkg_cur" = "x$pkg_new" ]
 }
@@ -610,19 +625,20 @@ database_build_inspect_dependencies ()
 # ============= Build and get a list of the complete recursive dependencies of a port =============
 database_build_get_complete_recursive_dependency ()
 {
-	local table origin dbpath dstfile
+	local table origin dbpath suffix dstfile
 	table=$1
 	origin=$2
+	suffix=$3
 	dbpath=${DBDIR}/requires/$origin
-	[ -e "$dbpath/$table.direct" ] || return 0
-	dstfile=$dbpath/$table.full
+	[ -e "$dbpath/$table.direct$suffix" ] || return 0
+	dstfile=$dbpath/$table.full$suffix
 	if [ ! -e "$dstfile" ]
 	then
 		while read origin_requirement
 		do
 			echo "$origin_requirement"
-			database_build_get_complete_recursive_dependency "$table" "$origin_requirement"
-		done < $dbpath/$table.direct | sort -u > $dstfile.tmp
+			database_build_get_complete_recursive_dependency "$table" "$origin_requirement" "$suffix"
+		done < $dbpath/$table.direct$suffix | sort -u > $dstfile.tmp
 		mv "$dstfile.tmp" "$dstfile"
 	fi
 	cat "$dstfile"
@@ -637,7 +653,7 @@ database_build_inspect_necessity_for_only_new_upgrade ()
 	origin=$1
 	level=$2
 	dbpath=${DBDIR}/requires/$origin
-	[ -e "$dbpath/necessary_port.${level}" ] && return
+	[ ! -d "$dbpath" -o -e "$dbpath/necessary_port.${level}" ] && return
 	tmpfile=${TMPDIR}/database_build_inspect_necessity_for_only_new_upgrade:`echo "$origin" | tr / :`
 	if database_query_does_a_port_need_update "$origin"
 	then
@@ -850,8 +866,10 @@ _database_build_reset_a_port_confdb ()
 			for tabel in requirements dependents
 			do
 				cat "${DBDIR}/$dbtag/$origin/$tabel.all.full" 2> /dev/null || :
+				cat "${DBDIR}/$dbtag/$origin/$tabel.all.full.orig" 2> /dev/null || :
 				rm -f "${DBDIR}/$dbtag/$origin/$tabel.run.full" \
-					"${DBDIR}/$dbtag/$origin/$tabel.build.full" 2> /dev/null
+					"${DBDIR}/$dbtag/$origin/$tabel.build.full" \
+					"${DBDIR}/$dbtag/$origin/is_customized" 2> /dev/null
 			done
 		done
 	} | sort -u >> ${DBDIR}/inspected_ports.update
@@ -930,6 +948,7 @@ database_build_post_inspect_dependencies ()
 		origin_regexp=`str_escape_regexp "$origin"`
 		origin_esc=`str_escape_replaceval "$origin"`
 		echo "s/^$origin_orig_regexp$/$origin_esc/" >> ${DBDIR}/REPLACE.complete_sed_pattern.tmp
+		[ -z "$origin_regexp" ] && continue
 		echo "s/^$origin_regexp$/$origin_orig_esc/" >> ${DBDIR}/REVERSE_REPLACE.complete_sed_pattern.tmp
 	done
 	mv "${DBDIR}/REPLACE.complete_sed_pattern.tmp" "${DBDIR}/REPLACE.complete_sed_pattern"
