@@ -2,13 +2,15 @@
 # ==============================================================================
 # portsreinstall library script
 # - Operations for queries to the temporary database -
-# Copyright (C) 2013 Mamoru Sakaue, MwGhennndo, All Rights Reserved.
+# Copyright (C) 2013-2014 Mamoru Sakaue, MwGhennndo, All Rights Reserved.
 # This software is distributed under the 2-Clause BSD License.
 # ==============================================================================
 
 # ============= Show a list of failed ports with their reasons =============
 database_query_show_list_failure ()
 {
+	local tmp_valid
+	tmp_valid=${TMPDIR}/database_query_show_list_failure:valid
 	if [ `cat "${DBDIR}/failed.list" 2> /dev/null | wc -l` -eq 0 ]
 	then
 		message_echo "INFO: No item is registered in this list."
@@ -19,7 +21,15 @@ database_query_show_list_failure ()
 	do
 		origin_regexp=`str_escape_regexp "$origin"`
 		grep -q -E "^$origin_regexp$" "${DBDIR}/stage.loop_list/ports_to_delete" 2> /dev/null \
-			&& continue
+			|| echo "$origin"
+	done > $tmp_valid
+	if [ `cat "$tmp_valid" 2> /dev/null | wc -l` -eq 0 ]
+	then
+		message_echo "INFO: No valid item is registered in this list."
+		return 1
+	fi
+	while read origin
+	do
 		note=`cat "${DBDIR}/notes/$origin/note_failtre" 2> /dev/null || :`
 		resolved=no
 		grep -q -E "^`str_escape_regexp \"$origin\"`$" \
@@ -47,7 +57,7 @@ database_query_show_list_failure ()
 		else
 			printf "%s\t%s\t%s\t%s\n" "$origin" "$pkgtag" "$note" "$resolved"
 		fi
-	done
+	done < $tmp_valid
 	:
 }
 
@@ -163,12 +173,14 @@ database_query_is_a_port_suppressed ()
 # ============= Check whether a port needs to be updated or upgraded =============
 database_query_does_a_port_need_update ()
 {
-	local origin dbpath
+	local origin dbpath new_version current_version
 	origin=$1
 	dbpath=${DBDIR}/requires/$origin
 	[ -e "$dbpath/conf_updated" ] && return
 	[ -e "$dbpath/new_version" ] || return
-	! diff "$dbpath/new_version" "$dbpath/current_version" > /dev/null 2>&1
+	new_version=`cat "$dbpath/new_version"`
+	current_version=`cat "$dbpath/current_version"`
+	[ "x$current_version" != "x$new_version" ]
 }
 
 # ============= Check before operations of a command which need the temporary database completely prepared =============
@@ -205,7 +217,7 @@ database_query_get_makevar_val ()
 # ============= Check whether configurations for a port is default =============
 database_query_is_default_conf ()
 {
-	local origin mode dbpath tmp_msg is_customized is_requiremnt_replaced files origin_regexp tmp_old tmp_new origin_requirement
+	local origin mode dbpath tmp_msg is_customized is_requiremnt_replaced files origin_regexp tmp_old tmp_new origin_requirement tmp_msg_customized
 	origin=$1
 	mode=$2
 	dbpath=${DBDIR}/requires/$origin
@@ -216,7 +228,7 @@ database_query_is_default_conf ()
 		is_customized=no
 		if [ `ls "${DBDIR}/conf/each_port/$origin" 2> /dev/null | wc -l` -gt 0 ]
 		then
-			files=`ls "${DBDIR}/conf/each_port/$origin" | str_cancat_items_for_sentence`
+			files=`ls "${DBDIR}/conf/each_port/$origin" | sed -E 's/^([^.]+).*/\1/' | str_concat_items_for_sentence`
 			echo "Knobs and miscellaneous customization by $files," >> $tmp_msg
 			is_customized=yes
 		fi
@@ -257,10 +269,10 @@ database_query_is_default_conf ()
 	[ `wc -c < $dbpath/is_customized` -eq 0 ] && return
 	if [ "x$mode" != xquiet ]
 	then
+		tmp_msg_customized=${TMPDIR}/database_query_is_default_conf:msg_customized
 		message_echo "INFO: This port is configured to be non-default because of"
-		message_cat 3<< eof
-`sed 's/^/         /' "$dbpath/is_customized"`
-eof
+		sed 's/^/         /' "$dbpath/is_customized" > $tmp_msg_customized
+		message_cat "$tmp_msg_customized"
 		message_echo "      so the prebuilt package is not used."
 	fi
 	return 1
