@@ -2,7 +2,7 @@
 # ==============================================================================
 # portsreinstall library script
 # - Deinstallation processes -
-# Copyright (C) 2013 Mamoru Sakaue, MwGhennndo, All Rights Reserved.
+# Copyright (C) 2013-2014 Mamoru Sakaue, MwGhennndo, All Rights Reserved.
 # This software is distributed under the 2-Clause BSD License.
 # ==============================================================================
 
@@ -71,6 +71,8 @@ deinstall_select_leaf_ports_to_delete ()
 			return 3
 		elif [ `cat "$src" 2> /dev/null | wc -l` -eq 0 ]
 		then
+			cp /dev/null "$dst_selected"
+			cp /dev/null "$dst_unselected"
 			return 2
 		fi
 		deinstall_select_leaf_ports_to_delete_dialog "$src" "$dst_selected" "$dst_unselected"
@@ -84,6 +86,9 @@ deinstall_select_leaf_ports_to_delete ()
 				cat "$dst_selected" "$dst_unselected" 2> /dev/null | sort -u > $tmp_srcprev
 				diff "$tmp_srcprev" "$src" > /dev/null 2>&1 || \
 					deinstall_select_leaf_ports_to_delete_dialog "$src" "$dst_selected" "$dst_unselected"
+			else
+				cp /dev/null "$dst_selected"
+				cp /dev/null "$dst_unselected"
 			fi
 			;;
 		all)
@@ -132,6 +137,8 @@ deinstall_select_obsolete_ports_to_delete ()
 	then
 		if [ `cat "$src" 2> /dev/null | wc -l` -eq 0 ]
 		then
+			cp /dev/null "$dst_selected"
+			cp /dev/null "$dst_unselected"
 			return 2
 		fi
 		deinstall_select_obsolete_ports_to_delete_dialog "$src" "$dst_selected" "$dst_unselected"
@@ -144,6 +151,9 @@ deinstall_select_obsolete_ports_to_delete ()
 				cat "$dst_selected" "$dst_unselected" 2> /dev/null | sort -u > $tmp_srcprev
 				diff "$tmp_srcprev" "$src" > /dev/null 2>&1 || \
 					deinstall_select_obsolete_ports_to_delete_dialog "$src" "$dst_selected" "$dst_unselected"
+			else
+				cp /dev/null "$dst_selected"
+				cp /dev/null "$dst_unselected"
 			fi
 			;;
 		all)
