@@ -2,7 +2,7 @@
 # ==============================================================================
 # portsreinstall library script
 # - Messages -
-# Copyright (C) 2013 Mamoru Sakaue, MwGhennndo, All Rights Reserved.
+# Copyright (C) 2013-2014 Mamoru Sakaue, MwGhennndo, All Rights Reserved.
 # This software is distributed under the 2-Clause BSD License.
 # ==============================================================================
 
@@ -63,18 +63,13 @@ message_echo ()
 }
 
 # ============= Output a message from stdin except for the batch mode =============
-# Input must be given via descriptor 3 if no file is given as the argument.
+# Use via pipe or redirection without argument is discouraged because it violates the screen size detection.
 message_cat ()
 {
 	local ncolumns
 	[ "x$opt_batch_mode" = xyes ] && return
 	ncolumns=`misc_get_console_column_size`
-	if [ $# -eq 0 ]
-	then
-		fold -s -w $ncolumns <&3
-	else
-		fold -s -w $ncolumns "$@"
-	fi
+	fold -s -w $ncolumns "$@"
 }
 
 # ============= Output a credit =============
@@ -87,7 +82,7 @@ message_credit ()
 	fold -s -w $ncolumns << eof
  -- Ports upgrading utility for massive forced reinstallation
  -- And for those who are pursuing the perfect packages environment
-Copyright (C) 2010 - 2013 Mamoru Sakaue, MwGhennndo, All Rights Reserved.
+Copyright (C) 2010 - 2014 Mamoru Sakaue, MwGhennndo, All Rights Reserved.
 Email: <sakaue.mamoru@samurai.mwghennn.net>
 Homepage: <http://www.mwghennndo.com/software/portsreinstall/>
 eof
