@@ -2,7 +2,7 @@
 # ==============================================================================
 # portsreinstall library script
 # - Reinstallation processes -
-# Copyright (C) 2013 Mamoru Sakaue, MwGhennndo, All Rights Reserved.
+# Copyright (C) 2013-2014 Mamoru Sakaue, MwGhennndo, All Rights Reserved.
 # This software is distributed under the 2-Clause BSD License.
 # ==============================================================================
 
@@ -222,7 +222,7 @@ reinstall_backup_and_delete_conflicts ()
 		printf '%s\t%s\n' "$origin" "$pkg" >> ${DBDIR}/deleted_conflicts
 		pkg_delete_f "$pkg" || \
 		{
-			message_echo "WARNING: Failed to deinstall $REINSTALL_CURRENTPKG by $PKGSYS_CMD_PKG_DELETE." >&2
+			message_echo "WARNING: Failed to deinstall $pkg by $PKGSYS_CMD_PKG_DELETE." >&2
 		}
 	done
 	cat "${DBDIR}/deleted_conflicts" 2> /dev/null | sort -u > ${DBDIR}/deleted_conflicts.tmp
@@ -299,6 +299,16 @@ reinstall_restore_conflicts ()
 	done < $tmpsrc
 }
 
+# ============= Check whether the all requirements are installed =============
+reinstall_are_requirements_ready ()
+{
+	[ -e "$REINSTALL_DBNODE_DIR/requirements.all.direct" ] || return 0
+	while read origin
+	do
+		pkg_info_eO "$origin" || return 1
+	done < $REINSTALL_DBNODE_DIR/requirements.all.direct
+}
+
 # ============= Get the all requirements ready for a port by restarting them if deinstalled =============
 reinstall_chk_and_restore_requirements ()
 {
@@ -311,19 +321,35 @@ reinstall_chk_and_restore_requirements ()
 		| while read origin
 	do
 		pkg_info_eO "$origin" && continue
+		origin_regexp=`str_escape_regexp "$origin"`
+		if grep -q -E "^$origin_regexp$" "${DBDIR}/taboo.all.list" 2> /dev/null
+		then
+			message_echo "INFO: Restoration of a requirement [$origin] is avoided because it is set taboo."
+			touch "$tmp_isfailed"
+			break
+		fi
 		origin_orig=`echo "$origin" \
 			| sed -E -f "${DBDIR}/REVERSE_REPLACE.complete_sed_pattern"`
-		[ "x$origin_orig" != "x$origin" ] \
-			&& pkg_info_eO "$origin_orig" && continue
+		if [ "x$origin_orig" != "x$origin" ]
+		then
+			pkg_info_eO "$origin_orig" && continue
+			origin_orig_regexp=`str_escape_regexp "$origin_orig"`
+			if grep -q -E "^$origin_orig_regexp$" "${DBDIR}/taboo.all.list" 2> /dev/null
+			then
+				message_echo "INFO: Restoration of a requirement [$origin_orig] is avoided because it is set taboo."
+				touch "$tmp_isfailed"
+				break
+			fi
+		fi
 		if ! pkgarc=`pkgsys_get_backup_pkg "$origin"` && \
 			! pkgarc=`pkgsys_get_backup_pkg "$origin_orig"`
 		then
-			origin_regexp=`str_escape_regexp "$origin"`
 			if grep -q -E "^$origin_regexp$" "${DBDIR}/failed.list" 2> /dev/null
 			then
 				touch "$tmp_isfailed"
 				break
 			fi
+			continue
 		fi
 		printf '%s\t%s\n' "$origin_orig" "$pkgarc" >> $tmp_restore
 	done
@@ -334,10 +360,10 @@ reinstall_chk_and_restore_requirements ()
 		pkg_regexp_esc=`str_escape_regexp "$pkg"`
 		if reinstall_chk_forbidden_conflicts "$pkg"
 		then
-			message_echo "INFO: Restoration of a conflict, $origin ($pkg), is avoided because it conflicts with installed packages."
+			message_echo "INFO: Restoration of a requirement [$origin ($pkg)] is avoided because it conflicts with installed packages."
 			continue
 		fi
-		message_echo "INFO: Restoring a backed-up requirement, $origin ($pkg)."
+		message_echo "INFO: Restoring a backed-up requirement [$origin ($pkg)]."
 		if pkg_add_fF "$pkgarc"
 		then
 			grep -v -E "[[:space:]]$pkg_regexp_esc$" "${DBDIR}/deleted_conflicts" \
@@ -440,9 +466,11 @@ reinstall_tell_update_to_depandents ()
 		for level in full direct
 		do
 			dbsuffix=${tag}.${level}
-			cat "$REINSTALL_DBNODE_DIR/dependents.$dbsuffix" 2> /dev/null \
+			cat "$REINSTALL_DBNODE_DIR/dependents.$dbsuffix" "$REINSTALL_DBNODE_DIR/dependents.$dbsuffix.orig" 2> /dev/null \
+				| sort -u \
 				| while read origin_dependent
 			do
+				[ -d "${DBDIR}/requires/$origin_dependent" ] || continue
 				touch "${DBDIR}/requires/$origin_dependent/need_reinstall_due_to_upgraded_requirements.$dbsuffix"
 				fileedit_rm_a_line "$origin_dependent" \
 					"${DBDIR}/success.$dbsuffix.list"
@@ -570,7 +598,7 @@ reinstall_exec ()
 		return
 	fi
 	reinstall_skip_if_in_a_list 'Ignored because being taboo' taboo.all.list && return
-	if ! reinstall_chk_and_restore_requirements
+	if ! reinstall_are_requirements_ready && ! reinstall_chk_and_restore_requirements
 	then
 		reinstall_restore_if_temporarily_deinstalled
 		for tag in all run build none
@@ -614,28 +642,38 @@ reinstall_exec ()
 		then
 			if ! reinstall_chk_stage FAILED_INSTALL_BY_PKG
 			then
-				func_pkg_inst_remote_fetch=pkg_inst_remote_fetch
-				func_pkg_inst_remote=pkg_inst_remote
-				if [ $opt_use_legacy_pkg_for_missing_pkgng = yes ]
+				if [ "x$REINSTALL_ORIGIN" = 'xports-mgmt/pkg'  -a "x$PKGSYS_USE_PKGNG" = xyes ]
 				then
-					func_pkg_inst_remote_fetch=pkg_inst_remote_wild_fetch
-					func_pkg_inst_remote=pkg_inst_remote_wild
-				fi
-				if $func_pkg_inst_remote_fetch "$REINSTALL_NEWPKGNAME"
-				then
-					pkgsys_get_conflicting_installed_pkgs install "$REINSTALL_ORIGIN" \
-						| reinstall_backup_and_delete_conflicts
-					reinstall_pkg_backup || :
-					reinstall_register_stage in_add_pkg
-					reinstall_deinstall
-					if $func_pkg_inst_remote "$REINSTALL_NEWPKGNAME"
+					if reinstall_deinstall && pkg_is_tool_available && pkg_info_e "$REINSTALL_NEWPKGNAME"
 					then
 						reinstall_register_stage_complete INSTALL_BY_PKG
 					else
 						reinstall_register_stage FAILED_INSTALL_BY_PKG
 					fi
 				else
-					reinstall_register_stage FAILED_INSTALL_BY_PKG
+					func_pkg_inst_remote_fetch=pkg_inst_remote_fetch
+					func_pkg_inst_remote=pkg_inst_remote
+					if [ $opt_use_legacy_pkg_for_missing_pkgng = yes ]
+					then
+						func_pkg_inst_remote_fetch=pkg_inst_remote_wild_fetch
+						func_pkg_inst_remote=pkg_inst_remote_wild
+					fi
+					if $func_pkg_inst_remote_fetch "$REINSTALL_NEWPKGNAME"
+					then
+						pkgsys_get_conflicting_installed_pkgs install "$REINSTALL_ORIGIN" \
+							| reinstall_backup_and_delete_conflicts
+						reinstall_pkg_backup || :
+						reinstall_register_stage in_add_pkg
+						reinstall_deinstall
+						if $func_pkg_inst_remote "$REINSTALL_NEWPKGNAME"
+						then
+							reinstall_register_stage_complete INSTALL_BY_PKG
+						else
+							reinstall_register_stage FAILED_INSTALL_BY_PKG
+						fi
+					else
+						reinstall_register_stage FAILED_INSTALL_BY_PKG
+					fi
 				fi
 			fi
 			if reinstall_chk_stage FAILED_IN_ADD_PKG && reinstall_chk_stage in_add_pkg
@@ -823,6 +861,27 @@ reinstall_exec ()
 		reinstall_register_stage_complete BUILD
 	fi
 	reinstall_deregister_stage in_build
+	if reinstall_chk_stage in_stage
+	then
+		message_echo "(Restarting the previously terminated staging process...)"
+		flag_restarted_stage=yes
+	else
+		reinstall_register_stage in_stage
+		flag_restarted_stage=no
+	fi
+	if ! reinstall_chk_stage_complete STAGE
+	then
+		if ! reinstall_make stage
+		then
+			reinstall_restore_conflicts "$REINSTALL_ORIGIN"
+			reinstall_restore_if_temporarily_deinstalled
+			reinstall_errproc 'STAGE operations'
+			temp_set_msg_current_stage "${_MSG_CURRENT_STAGE_general}"
+			return
+		fi
+		reinstall_register_stage_complete STAGE
+	fi
+	reinstall_deregister_stage in_stage
 	if reinstall_chk_stage in_install
 	then
 		message_echo "(Restarting the previously terminated installation process...)"
@@ -849,11 +908,22 @@ reinstall_exec ()
 		fi
 		reinstall_register_stage_complete BEFOREDEINSTALL
 	fi
-	pkgsys_get_conflicting_installed_pkgs install "$REINSTALL_ORIGIN" \
-		| reinstall_backup_and_delete_conflicts
 	reinstall_deinstall
 	if ! reinstall_chk_stage_complete INSTALL
 	then
+		message_echo "(Checking installation conflict...)"
+		stagedir=`database_query_get_makevar_val "$REINSTALL_ORIGIN" STAGEDIR`
+		{
+			( set -e
+				cd "$stagedir"
+				find . -not -type d
+			) | sed 's|^\.||' | while read filepath
+			do
+				[ -e "$filepath" ] || continue
+				pkg_which "$filepath"
+			done
+			pkgsys_get_conflicting_installed_pkgs install "$REINSTALL_ORIGIN"
+		} | sort -u | reinstall_backup_and_delete_conflicts
 		if ! reinstall_chk_stage FAILED_INSTALL
 		then
 			if reinstall_make $insttarget || \
