@@ -2,7 +2,7 @@
 # ==============================================================================
 # portsreinstall library script
 # - String processing -
-# Copyright (C) 2013 Mamoru Sakaue, MwGhennndo, All Rights Reserved.
+# Copyright (C) 2013-2014 Mamoru Sakaue, MwGhennndo, All Rights Reserved.
 # This software is distributed under the 2-Clause BSD License.
 # ==============================================================================
 
@@ -68,7 +68,7 @@ str_dirpath_to_origin ()
 }
 
 # ============= Concatenate a space/linefeed separated list to a apart of sentence =============
-str_cancat_items_for_sentence ()
+str_concat_items_for_sentence ()
 {
 	sed 's/$/,/' | tr '\n' ' ' | sed -E 's/, $//;s/, ([^ ]*)$/ and \1/'
 }
