@@ -2,7 +2,7 @@
 # ==============================================================================
 # portsreinstall library script
 # - Help messages -
-# Copyright (C) 2013 Mamoru Sakaue, MwGhennndo, All Rights Reserved.
+# Copyright (C) 2013-2014 Mamoru Sakaue, MwGhennndo, All Rights Reserved.
 # This software is distributed under the 2-Clause BSD License.
 # ==============================================================================
 
@@ -60,10 +60,11 @@ USAGE: ${APPNAME} [OPTIONS] [--] [command]
           | clean [normal] | clean force | reset [all] | reset keepopts
           | ok add globs... | ok del globs...
           | taboo add globs... | taboo del globs...
+          | need add globs... | need del globs...
           | noneed add globs... | noneed del globs...
           | reselect leaves | reselect obsolete
           | save [dir] | load path | glob globs... | options
-          | reconf globs... | forget globs...
+          | rmconf globs... | reconf globs... | forget globs...
           | escape globs... | restore globs...
           | pkgsanity [globs...]
           | show todo | show done | show resolved | show failure | show redo
